from typing import Type

from httpmessageprocessorutil.automapped import AutoMappedMessageProcessor

AnyType = Type["AnyType"]


class AutoMappedWebApi:
    def __init__(
        self,
        interactor_type: AnyType,
        request_type: Type = None,
        processor_type: Type[AutoMappedMessageProcessor] = None,
        success_status: int = 200,
        exception_map: dict = None,
        extra_mapping: dict = None,
        event_name: str = None,
        response_content_type: str = None,
    ):
        self.interactor_type = interactor_type
        self.request_type = request_type
        self.success_status = success_status
        self.exception_map = exception_map
        self.extra_mapping = extra_mapping
        self.event_name = event_name
        self.response_content_type = response_content_type
        self.processor_type = processor_type or AutoMappedMessageProcessor

    def __str__(self):
        values = [
            f"Processor: {self.processor_type}",
            f"Interactor: {self.interactor_type}",
            f"Request: {self.request_type}",
        ]
        return " - ".join(values)



from ._AutoMappedDiRouterProcessorBuilder import AutoMappedDiRouterProcessorBuilder
from ._AutoMappedRouteBuilder import AutoMappedRouteBuilder
from ._AutoMappedWebApi import AutoMappedWebApi
from ._DiRouterProcessorBuilder import DiRouterProcessorBuilder



from setuptools import setup, find_packages
setup(
      name="DiHttpMessageProcessor",
      version="0.2.2",
      packages=find_packages(where="."),
      install_requires=[
            'HttpMessageProcessor==0.12.1',
            'EdpDependencyInjection==0.2.1',
      ],
      url="http://www.e-deploy.com.br",
      author="Eduardo Almeida",
      author_email="eduardo.almeida@e-deploy.com.br"
)



from typing import Dict, Type

from dependencyinjection import DependencyContext
from httpmessageprocessor import RouterProcessorBuilder, RouteNotMapped
from messageprocessor import MessageProcessor
from simplehttprouter import Route, SimpleHttpRouter


class DiRouterProcessorBuilder(RouterProcessorBuilder):
    def __init__(
        self,
        dependency_context: DependencyContext,
        route_definition: Dict[Route, Type[MessageProcessor]]
    ):
        self.dependency_context = dependency_context
        self.route_definition = route_definition

        self.router = SimpleHttpRouter(list(self.route_definition.keys()))

    def get_processor(self, route):
        found_route = self.router.get_route(route)
        if found_route is None:
            raise RouteNotMapped(route.method, route.path)

        processor_cls = self.route_definition[found_route.route]
        processor_instance = self.dependency_context.get_instance(processor_cls)
        return found_route, processor_instance



from typing import Dict, Type, Union

from simplehttprouter import Route, SimpleHttpRouter

from dependencyinjection import DependencyContext
from httpmessageprocessor import RouteNotMapped
from messageprocessor import MessageProcessor
from ._AutoMappedRouteBuilder import AutoMappedRouteBuilder
from ._AutoMappedWebApi import AutoMappedWebApi
from ._DiRouterProcessorBuilder import DiRouterProcessorBuilder


class AutoMappedDiRouterProcessorBuilder(DiRouterProcessorBuilder):
    def __init__(
        self,
        dependency_context: DependencyContext,
        route_definition: Dict[Route, Union[Type[MessageProcessor], AutoMappedWebApi]] = None,
    ):
        self.auto_mapped_definition: Dict[Route, AutoMappedWebApi] = {}
        self.processor_mapped_definition: Dict[Route, Type[MessageProcessor]] = {}
        self.dependency_context = dependency_context

        self._build_routes_definitions(route_definition)
        self._get_builder_dependency_routes()

        super().__init__(dependency_context, self.processor_mapped_definition)
        self.auto_mapped_router = SimpleHttpRouter(list(self.auto_mapped_definition.keys()))

    def _build_routes_definitions(self, route_definition):
        if route_definition is not None:
            for route, value in route_definition.items():
                if isinstance(value, AutoMappedWebApi):
                    self.auto_mapped_definition[route] = value
                else:
                    self.processor_mapped_definition[route] = value

    def _get_builder_dependency_routes(self):
        builder_types = self.dependency_context.get_dependencies_by_parent(AutoMappedRouteBuilder)
        for builder_type in builder_types:
            builder_instance = self.dependency_context.get_instance(builder_type)
            self._build_routes_definitions(builder_instance.build_routes())

    def get_processor(self, route):
        found_route = self.router.get_route(route)
        if found_route is not None:
            return super().get_processor(route)

        found_route = self.auto_mapped_router.get_route(route)
        if found_route is not None:
            definition = self.auto_mapped_definition[found_route.route]

            interactor = self.dependency_context.get_instance(definition.interactor_type)
            processor = definition.processor_type(
                request_type=definition.request_type,
                interactor=interactor,
                exception_map=definition.exception_map,
                success_status=definition.success_status,
                extra_mapping=definition.extra_mapping,
                event_name=definition.event_name,
                response_content_type=definition.response_content_type,
            )
            return found_route, processor

        raise RouteNotMapped(route.method, route.path)
